import type { RawThemeSetting, ThemeRegistrationAny } from 'shiki'
import transform, { type StyleTuple } from 'css-to-react-native'
import { StyleSheet, type TextStyle } from 'react-native'

export interface HighlighterStyleSheet {
  container: TextStyle
  token: TextStyle
  selection: TextStyle
  styles: Record<string, TextStyle>
  [key: string]: TextStyle | Record<string, TextStyle>
}

// Allowed CSS properties for conversion
const ALLOWED_STYLE_PROPERTIES: Record<string, boolean> = {
  color: true,
  backgroundColor: true,
  fontWeight: true,
  fontStyle: true,
}

// Cleans and transforms CSS properties to React Native compatible style
function cleanStyle(style: Record<string, any>): TextStyle {
  const filteredStyles = Object.entries(style)
    .filter(([key]) => ALLOWED_STYLE_PROPERTIES[key])
    .map<StyleTuple>(([key, value]) => [key, value])

  return transform(filteredStyles)
}

function processSettings(settings: RawThemeSetting[]): Record<string, TextStyle> {
  const tokenStyles: Record<string, TextStyle> = {}

  settings.forEach((setting) => {
    if (!setting.scope || !setting.settings)
      return

    const style = cleanStyle(setting.settings)

    // Handle single scope
    if (typeof setting.scope === 'string') {
      tokenStyles[setting.scope] = style
    }
    // Handle multiple scopes
    else if (Array.isArray(setting.scope)) {
      setting.scope.forEach((scope) => {
        tokenStyles[scope] = style
      })
    }
  })

  return tokenStyles
}

export function convertShikiTheme(theme: ThemeRegistrationAny): ReturnType<typeof StyleSheet.create> {
  // Get all styles from both settings and tokenColors
  const settings = [...(theme.settings || []), ...(theme.tokenColors || [])]

  // Process basic colors from the theme
  const backgroundColor = theme.colors?.['editor.background'] || theme.bg || '#1e1e1e'
  const defaultColor = theme.colors?.['editor.foreground'] || theme.fg || '#d4d4d4'
  const selectionColor = theme.colors?.['editor.selectionBackground'] || 'rgba(255, 255, 255, 0.1)'

  // Process token styles
  const tokenStyles = processSettings(settings)

  // Convert token styles to RN styles
  const tokenStylesMap = Object.entries(tokenStyles).reduce<Record<string, TextStyle>>((acc, [scope, style]) => {
    acc[`token-${scope.replace(/\./g, '-')}`] = style
    return acc
  }, {})

  // Create the complete styles object
  const completeStyles: HighlighterStyleSheet = {
    container: {
      backgroundColor,
      flex: 1,
      padding: 16,
    },
    token: {
      color: defaultColor,
      fontSize: 14,
      lineHeight: 21,
    },
    selection: {
      backgroundColor: selectionColor,
    },
    styles: tokenStylesMap,
  }

  return StyleSheet.create(completeStyles)
}
